/**
 * Copyright (C) 2016 OGIS-RI Co., Ltd.
 * 
 * This software is released under the MIT License.
 * http://opensource.org/licenses/mit-license.php
 */

var express = require('express');
var router = express.Router();

var passport = require('passport');
var aws = require('aws-sdk');
var sts = new aws.STS();
var request = require('request');
var fs = require('fs');

router.get('/login', function(req, res, next){
  res.render('login', {
    title: "ログイン前画面"
  });
});

router.post('/login', passport.authenticate('openidconnect'));

router.get('/cb', passport.authenticate('openidconnect', {failureRedirect: '/'}),
  function(req, res) {
    res.cookie('express',
      {
        authInfo: req.authInfo,
        userInfo: req.user._json
      },
      {
        secure: true,
        httpOnly: true
      }
    );
    var departmentNumber = req.user._json.departmentNumber;
    res.render(
      "choose",
      {
        // isAdmin()は引数が管理ユーザーの部門番号(000000)であることを確認する
        admin: isAdmin(departmentNumber)
      }
    );
  }
);

router.post('/ec2/:id', function(req, res, next) {
  var cookie = req.cookies.express;
  if(cookie){
    var policyRequest = req.params.id;
    var AWS_ACCOUNT = ""; //AWS Account ID
    var EC2_FULLACCESS_POLICY = fs.readFileSync("./conf/ec2_fullaccess_policy.json", "utf-8"); 
    var EC2_DESCRIBE_POLICY = fs.readFileSync("./conf/ec2_describe_policy.json", "utf-8"); 
    var idToken = cookie.authInfo.id_token;
    var iss = JSON.parse(new Buffer(idToken.split(".")[1], "base64").toString()).iss;
    var departmentNumber = cookie.userInfo.departmentNumber;
    var admin = isAdmin(departmentNumber)

    // ID連携ユーザー向けのサインインURLの取得を行うためのエンドポイント
    var SIGNIN_URL = "https://signin.aws.amazon.com/federation";
    // サインイン後にアクセスするURL
    var CONSOLE_URL = "https://ap-northeast-1.console.aws.amazon.com/ec2";
    // サインイン後に引き受ける権限の基本設定
    var params = {
      RoleArn: 'arn:aws:iam::' + AWS_ACCOUNT + ':role/IAMRoleForEC2',
      DurationSeconds: 900,
    };
    // 選択された権限に応じたポリシーをパラメータに設定する
    if (admin && policyRequest == "fullaccess") {
      //管理ユーザー且つFull Access権限を要求した場合
      params.RoleSessionName = "EC2-FULLACCESS-POLICY";
      params.Policy = EC2_FULLACCESS_POLICY;
    } else {
      //それ以外のユーザーの場合
      params.RoleSessionName = "EC2-DESCRIBE-POLICY";
      params.Policy = EC2_DESCRIBE_POLICY;
    }
    
    // 選択された権限でサインインが可能なURLを生成する
    sts.assumeRole(params, function(err, data) {
    if (err) {
      // Error処理を記述
      } else {
        // サインインURL取得に必要な要求を作成する
        var sessionJson = {
          "sessionId": data.Credentials.AccessKeyId,
          "sessionKey": data.Credentials.SecretAccessKey,
          "sessionToken": data.Credentials.SessionToken
        };
        var session = encodeURIComponent(JSON.stringify(sessionJson))
        var signinParam = "?Action=getSigninToken"
                        + "&SessionType=json"
                        + "&Session="+session;
        var options = {
          url: SIGNIN_URL + signinParam,
          json: true
        };
        // サインインURLの作成し、リダイレクト応答をする
        request.get(options, function (error, response, body) {
          var signinToken = body.SigninToken;
          var issuer = iss;
          var redirectQuery = "?Action=login"
                            + "&SigninToken="+encodeURIComponent(signinToken)
                            + "&Issuer="+encodeURIComponent(iss)
                            + "&Destination="+encodeURIComponent(CONSOLE_URL);
          var redirect = SIGNIN_URL + redirectQuery
          res.redirect(redirect);
        });
      }
    });
  } else {
    // Error処理を記述
  }
});

function isAdmin(departmentNumber){
  if(departmentNumber == "000000")return true;
  else return false;
}

module.exports = router;
